<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/
class BOOKMARKS_BOL_Service
{
    CONST LIST_LATEST = 'latest';
    CONST LIST_ONLINE = 'online';
    CONST LIST_NOTIFY = 'notify';
    
    CONST COUNT_USER_IN_NOTIFY = 7;
    CONST COUNT_CRON_USER = 100;
    
    private static $classInstance;

    /**
     * 
     * @return BOOKMARKS_BOL_Service
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }
    /* @var $markDto BOOKMARKS_BOL_MarkDao */
    private $markDto;
    private $notifyLogDao;

    private function __construct()
    {
        $this->markDto = BOOKMARKS_BOL_MarkDao::getInstance();
        $this->notifyLogDao = BOOKMARKS_BOL_NotifyLogDao::getInstance();
    }
    
    public function isMarked( $userId, $markUserId )
    {
        return $this->markDto->isMarked($userId, $markUserId);
    }
    
    public function mark( $userId, $markUserId )
    {
        return $this->markDto->mark($userId, $markUserId);
    }
    
    public function unmark( $userId, $markUserId )
    {
        return $this->markDto->unmark($userId, $markUserId);
    }
    
    public function findBookmarksCount( $userId, $list = self::LIST_LATEST )
    {
        return $this->markDto->findBookmarksCount($userId, $list);
    }
    
    public function findBookmarksUserIdList( $userId, $first, $count, $list = self::LIST_LATEST )
    {
        return $this->markDto->findBookmarksUserIdList($userId, $first, $count, $list);
    }
    
    public function cleareExpiredNotifyLog( $timestamp )
    {
        return $this->notifyLogDao->cleareExpiredNotifyLog($timestamp);
    }

    public function findUserIdListForNotify( $timeStamp, $first, $count )
    {
        return $this->notifyLogDao->findUserIdListForNotify($timeStamp, $first, $count);
    }
    
    public function getMarkedListByUserId( $userId, $userIdList )
    {
        return $this->markDto->getMarkedListByUserId($userId, $userIdList);
    }
    
    public function sendNotifyForUser( $userId )
    {
        if ( empty($userId) )
        {
            return FALSE;
        }
        
        $bookmarkUserIdList = $this->markDto->findBookmarksUserIdList($userId, 0, self::COUNT_USER_IN_NOTIFY, self::LIST_NOTIFY);
        
        if ( empty($bookmarkUserIdList) )
        {
            return TRUE;
        }
        
        $cmp = new BOOKMARKS_CMP_Notify($userId, $bookmarkUserIdList);
        $cmp->sendNotification();
        
        return TRUE;
    }
    
    public function notifyLogSave( $userId )
    {
        $this->notifyLogDao->notifyLogSave($userId);
    }
}
