<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

class GOOGLELOCATION_CMP_MapEntityList extends OW_Component
{
    protected $label;
    protected $url = '';
    protected $IdList = array();
    protected $display = false;
    protected $count = 0;
    protected $backUri = null;
    
    const DISPLAY_COUNT = 5;

    public function __construct( $IdList, $lat, $lng, $backUri = null )
    {
        $this->IdList = $IdList;
        $this->lat = (float)$lat;
        $this->lng = (float)$lng;
        $this->count = count($IdList);

        $this->setBackUri($backUri);
        
        if ( count($IdList) > self::DISPLAY_COUNT )
        {
            $hash = GOOGLELOCATION_BOL_LocationService::getInstance()->saveEntityListToSession($IdList);

            $this->display = true;
            $this->label = OW::getLanguage()->text('googlelocation', 'map_user_list_view_all_button_label', array( 'count' => count($IdList) ) );
            $this->url = ow::getRouter()->urlForRoute('googlelocation_user_list', array( 'lat' => $this->lat, 'lng' => $this->lng, 'hash' => $hash ) );
        }
        
        parent::__construct();

        $this->template = OW::getPluginManager()->getPlugin('googlelocation')->getCmpViewDir() . 'map_entity_list.html';
    }

    public function getBackUrl()
    {
        return OW_URL_HOME . $this->backUri;
    }

    public function getBackUri()
    {
        return $this->backUri;
    }

    public function setBackUri( $uri )
    {
        $this->backUri = $uri;
    }

    public function setViewMoreUrl( $url )
    {
        $this->url = $url;
    }
    
    public function setViewMoreLabel( $label )
    {
        $this->label = $label;
    }
    
    public function setDisplayViewMoreButton( $display )
    {
        $this->display = $display;
    }

    public function onBeforeRender()
    {
        parent::onBeforeRender();

        if ( !empty($this->url) && !empty($this->backUri) )
        {
            $this->url = OW::getRequest()->buildUrlQueryString( $this->url, array( 'backUri' => $this->getBackUri() ) );
        }
        
        $this->addComponent('entityList', $this->getListCmp());
        $this->assign('url', $this->url);
        $this->assign('viewAllLabel', $this->label);
        $this->assign('displayViewAllButton', $this->display);
    }
    
    protected function getListCmp()
    {
        $new = new BASE_CMP_MiniAvatarUserList(array_slice($this->IdList, 0, self::DISPLAY_COUNT));
        
        switch(true)
        {
            case $this->count <= 8:
                    $new->setCustomCssClass('ow_big_avatar');
                break;
            default:
                    //$new->setCustomCssClass(BASE_CMP_MiniAvatarUserList::CSS_CLASS_MINI_AVATAR);
                break;
        }
        
        return $new;
    }
}