<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
final class OXDATE_BOL_LicenceService
{
    /**
     * Singleton instance.
     *
     * @var OXDATE_BOL_LicenceService
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return OXDATE_BOL_Service
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Constructor.
     */
    private function __construct()
    {
        
    }

    public function validateKey()
    {
        $licenseKey = OW::getConfig()->getValue('oxdate', 'license_key');

        try
        {
            $result = $this->checkLicenseKey($licenseKey);
        }
        catch ( LogicException $e )
        {
            // if server returned invalid responce, add result to log and exit
            OW::getLogger()->addEntry($e->getMessage());
            return;
        }

        if ( isset($result['licenseValid']) )
        {
            OW::getConfig()->saveConfig('oxdate', 'license_key_valid', (bool) $result['licenseValid']);
        }

        if ( isset($result['brandingRemoval']) )
        {
            OW::getConfig()->saveConfig('oxdate', 'brand_removal', (bool) $result['brandingRemoval']);
        }

        OW::getConfig()->saveConfig('oxdate', 'license_info', json_encode($result));
        
        // update license status for all dependent items
        BOL_StorageService::getInstance()->checkUpdates();
    }

    /**
     * @param string $licenseKey
     * @throws LogicException
     * @return mixed
     */
    public function checkLicenseKey( $licenseKey )
    {
        $url = parse_url(OW_URL_HOME);
        $paramsString = json_encode(array('bundleName' => 'oxdate', 'licenseKey' => $licenseKey, 'domain' => $url['host'], 'dir' => OW_DIR_ROOT));
        $url = BOL_PluginService::UPDATE_SERVER . 'validate-bundle-license-key?params=' . urlencode($paramsString);

        $data = file_get_contents($url);
        $result = $data ? json_decode($data, true) : array();
        
        if ( !isset($result['result']) )
        {
            throw new LogicException("Invalid server response");
        }

        return $result;
    }
}
