<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class PROTECTEDPHOTOS_CLASS_QueryHandler
{
    private static $instance;

    public static function getInstance()
    {
        if ( null === self::$instance )
        {
            self::$instance = new self();
        }

        return self::$instance;
    }

    private $service;

    private function __construct()
    {
        $this->service = PROTECTEDPHOTOS_BOL_Service::getInstance();
    }

    public function init()
    {
        $em = OW::getEventManager();

        $em->bind('photo.getPhotoList', array($this, 'handlerQueryGetPhotoList'));
        $em->bind('base.query.content_filter', array($this, 'handlerBaseContentFilter'));
        $em->bind('photo.getPhotoList', array($this, 'handlerQueryUserPhoto'));
    }

    public function unbind()
    {
        OW::getEventManager()->unbind('photo.getPhotoList', array($this, 'handlerQueryGetPhotoList'));
        OW::getEventManager()->unbind('base.query.content_filter', array($this, 'handlerBaseContentFilter'));
        OW::getEventManager()->unbind('photo.getPhotoList', array($this, 'handlerQueryUserPhoto'));
    }

    public function handlerQueryGetPhotoList( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        if ( OW::getUser()->isAuthorized('photo') ) return;

        $params = $builderEvent->getParams();

        switch ( true )
        {
            case in_array($params['listType'], array(
                'findAlbumPhotoList.latest',
                'findEntityPhotoList'
            ), true):
                $this->handlerQueryForPhoto($builderEvent);
                break;
            case in_array($params['listType'], array(
                'latest',
                'featured',
                'toprated',

                'findPhotoListByUserId',
                'searchByDesc',
                'searchByUser',
                'searchByHashtag',
                'searchByUsername'
            ), true):
                $this->handlerQueryForPublicList($builderEvent);
                break;
        }
    }

    private function handlerQueryForPhoto( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();
        $aliases = $params['aliases'];

        $sql = 'LEFT JOIN `%s` AS `pwd` ON(`%s`.`albumId` = `pwd`.`albumId`)';
        $builderEvent->addJoin(sprintf($sql, PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName(), $aliases['photo']));
        $builderEvent->addWhere('`pwd`.`albumId` IS NULL');
    }

    private function handlerQueryForPublicList( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();
        $aliases = $params['aliases'];

        $sql = 'LEFT JOIN `%s` AS `pwd` ON(`%s`.`albumId` = `pwd`.`albumId`)';
        $builderEvent->addJoin(sprintf($sql, PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName(), $aliases['photo']));
        $builderEvent->addWhere(sprintf('`pwd`.`albumId` IS NULL OR `pwd`.`privacy` IN(%s)',
            OW::getDbo()->mergeInClause(array(
                PROTECTEDPHOTOS_BOL_Service::PRIVACY_PUBLIC,
                PROTECTEDPHOTOS_BOL_Service::PRIVACY_PASSWORD
            ))
        ));
    }

    public function handlerBaseContentFilter( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();

        switch ( $params['method'] )
        {
            case 'BOL_RateDao::findMostRatedEntityList':
                $this->handlerRateFilter($builderEvent);
                break;
            case 'BOL_CommentDao::findMostCommentedEntityList':
                $this->handlerCommentFilter($builderEvent);
                break;
        }
    }

    private function handlerRateFilter( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();

        $builderEvent->addJoin(sprintf(
            'INNER JOIN `%s` as `p` ON(`p`.`id` = `%s`.`entityId`)',
            OW_DB_PREFIX . 'photo',
            $params['tables']['content']
        ));
        $builderEvent->addJoin(sprintf(
            'LEFT JOIN `%s` AS `pwd` ON(`p`.`albumId` = `pwd`.`albumId`)',
            PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName()
        ));
        $builderEvent->addWhere(sprintf('`pwd`.`albumId` IS NULL OR `pwd`.`privacy` IN(%s)',
            OW::getDbo()->mergeInClause(array(
                PROTECTEDPHOTOS_BOL_Service::PRIVACY_PUBLIC,
                PROTECTEDPHOTOS_BOL_Service::PRIVACY_PASSWORD
            ))
        ));
    }

    private function handlerCommentFilter( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();

        $builderEvent->addJoin(sprintf(
            'INNER JOIN `%s` as `p` ON(`p`.`id` = `%s`.`entityId`)',
            OW_DB_PREFIX . 'photo',
            $params['tables']['comment_entity']
        ));
        $builderEvent->addJoin(sprintf(
            'LEFT JOIN `%s` AS `pwd` ON(`p`.`albumId` = `pwd`.`albumId`)',
            PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName()
        ));
        $builderEvent->addWhere(sprintf('`pwd`.`albumId` IS NULL OR `pwd`.`privacy` IN(%s)',
            OW::getDbo()->mergeInClause(array(
                PROTECTEDPHOTOS_BOL_Service::PRIVACY_PUBLIC,
                PROTECTEDPHOTOS_BOL_Service::PRIVACY_PASSWORD
            ))
        ));
    }

    public function handlerQueryUserPhoto( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();

        switch ( true )
        {
            case in_array($params['listType'], array(
                'getUserAlbumList',
                'findUserAlbumList'
            ), true):
                $this->handlerUserAlbumList($builderEvent);
                break;
        }
    }

    private function handlerUserAlbumList( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        if ( OW::getUser()->isAuthorized('photo') ) return;

        $params = $builderEvent->getParams();
        $aliases = $params['aliases'];

        if ( OW::getUser()->isAuthenticated() )
        {
            $builderEvent->addJoin(sprintf(
                'LEFT JOIN `%s` AS `pwd` ON(`%s`.`id` = `pwd`.`albumId`)',
                PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName(),
                $aliases['album']
            ));
            $builderEvent->addJoin(sprintf(
                'LEFT JOIN `%s` AS `access` ON(`access`.`passwordId` = `pwd`.`id`)',
                PROTECTEDPHOTOS_BOL_AccessDao::getInstance()->getTableName()
            ));
            $builderEvent->addWhere(sprintf('(`pwd`.`albumId` IS NULL OR `access`.`passwordId` IS NULL OR
                (`pwd`.`privacy` IN(%s) AND `access`.`userId` = :pppUserId)) OR (`pwd`.`privacy` IN (%s))',
                OW::getDbo()->mergeInClause(array(
                    PROTECTEDPHOTOS_BOL_Service::PRIVACY_FRIENDS_ONLY,
                    PROTECTEDPHOTOS_BOL_Service::PRIVACY_INDIVIDUAL_FRIENDS,
                    PROTECTEDPHOTOS_BOL_Service::PRIVACY_PASSWORD
                )),
                OW::getDbo()->mergeInClause(array(
                    PROTECTEDPHOTOS_BOL_Service::PRIVACY_PASSWORD,
                    PROTECTEDPHOTOS_BOL_Service::PRIVACY_PUBLIC
                ))
            ));
            $builderEvent->addQueryParam('pppUserId', OW::getUser()->getId());
        }
        else
        {
            $builderEvent->addJoin(sprintf(
                'LEFT JOIN `%s` AS `pwd` ON(`%s`.`id` = `pwd`.`albumId`)',
                PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName(),
                $aliases['album']
            ));
            $builderEvent->addWhere(sprintf('`pwd`.`albumId` IS NULL OR `pwd`.`privacy` IN(%s)',
                OW::getDbo()->mergeInClause(array(
                    PROTECTEDPHOTOS_BOL_Service::PRIVACY_PUBLIC,
                    PROTECTEDPHOTOS_BOL_Service::PRIVACY_PASSWORD
                ))
            ));
        }
    }
}
